#include "meindatatype.h"
#include "maxon/datadescription_ui.h"
#include "customgui_meindatatype.h"
#include "lib_description.h"

namespace ace
{

MeinDataType::MeinDataType(Float value) : _value(value), _TimeScaler(1.0)//, _gradient(*Gradient::Alloc())
{
	_gradient = Gradient::Alloc();
	_spline= SplineData::Alloc();
}
MeinDataType::~MeinDataType()
{
	//SplineData::Free(_spline);

}

maxon::Result<void> MeinDataType::DescribeIO(const maxon::DataSerializeInterface& stream)
{
	iferr_scope;

	PrepareDescribe(stream, MeinDataType);

	Describe("_value", _value, Float, maxon::DESCRIBEFLAGS::NONE) iferr_return;

	return maxon::OK;
}

Float MeinDataType::GetValue() const
{
	return _value;
}

void MeinDataType::SetValue(Float value)
{
	_value = value;
}

MAXON_DATATYPE_REGISTER(MeinDataType);
MAXON_DEPENDENCY_REGISTER(MeinDataTypeRegistration);

Int32 MeinDataTypeClass::GetId()
{
	return MEINDATATYPE_ID;
}

CustomDataType* MeinDataTypeClass::AllocData()
{
	MeinDataType* data = NewObjClear(MeinDataType);
	return data;
}

void MeinDataTypeClass::FreeData(CustomDataType* data)
{
	DeleteObj(data);
}

Bool MeinDataTypeClass::CopyData(const CustomDataType* src, CustomDataType* dst, AliasTrans* aliastrans)
{
	const MeinDataType& srcData = *static_cast<const MeinDataType*>(src);
	MeinDataType& dstData = *static_cast<MeinDataType*>(dst);
	dstData = srcData;
	return true;
}

Int32 MeinDataTypeClass::Compare(const CustomDataType* a, const CustomDataType* b)
{
	const	MeinDataType& dataA = *static_cast<const	MeinDataType*>(a);
	const	MeinDataType& dataB = *static_cast<const	MeinDataType*>(b);

	if (dataA == dataB)
		return 0;
	if (dataA < dataB)
		return -1;
	return 1;
}

Bool MeinDataTypeClass::WriteData(const CustomDataType* t_d, HyperFile* hf)
{
	const MeinDataType& data = *static_cast<const MeinDataType*>(t_d);
	return hf->WriteFloat(data.GetValue());
}

Bool MeinDataTypeClass::ReadData(CustomDataType* t_d, HyperFile* hf, Int32 level)
{
	MeinDataType& data = *static_cast<MeinDataType*>(t_d);

	if (level > 0)
	{
		Float value;
		const Bool readSuccess = hf->ReadFloat(&value);
		if (readSuccess == false)
			return false;
		data.SetValue(value);
	}
	return true;
}

Bool MeinDataTypeClass::InterpolateKeys(GeData& res, const GeData& t_dataA, const GeData& t_dataB, Float mix, Int32 flags)
{
	const MeinDataType& dataA = *static_cast<const MeinDataType*>(t_dataA.GetCustomDataType(MEINDATATYPE_ID));
	const MeinDataType& dataB = *static_cast<const MeinDataType*>(t_dataB.GetCustomDataType(MEINDATATYPE_ID));

	const Float valueA = dataA.GetValue();
	const Float valueB = dataB.GetValue();
	const MeinDataType mixedValue = MeinDataType(maxon::Blend(valueA, valueB, mix));
	res = GeData(MEINDATATYPE_ID, (CustomDataType&)mixedValue);
	return true;
}

const Char* MeinDataTypeClass::GetResourceSym()
{
	return "MEINDATATYPE";
}

void MeinDataTypeClass::GetDefaultProperties(BaseContainer& data)
{
	data.SetInt32(DESC_CUSTOMGUI, MEINDATATYPEGUI_ID);
	data.SetInt32(DESC_ANIMATE, DESC_ANIMATE_ON);
}

maxon::Result<void> MeinDataTypeClass::RegisterDataType()
{
	iferr_scope;

	static const Int32 version = 1;
	const Bool isDataTypeRegistered = RegisterCustomDataTypePlugin("MeinDataType"_s, CUSTOMDATATYPE_INFO_LOADSAVE, NewObjClear(MeinDataTypeClass), version);
	CheckState(isDataTypeRegistered == true);

	return maxon::OK;
}

} // namespace ace